package me.km.chatchannel;

import java.util.HashSet;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import me.km.KajetansMod;
import me.km.permissions.Permissions;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.text.TextComponentString;

public class ChatChannel 
{
    // -------------------------------------------------------------------------
    // static stuff
    // -------------------------------------------------------------------------
    
    public static ChatChannel getDevChannel()
    {
        return DEV_CHANNEL;
    };
    
    private final static ChatChannel DEV_CHANNEL = new ChatChannel("Dev", TextColor.LIGHT_PURPLE, 
            (cc, s) -> 
            {
                TextComponentString text = new TextComponentString(s);
                cc.receivers.forEach(p -> p.sendMessage(text));
            },
            (cc, p) -> 
            {
                if(KajetansMod.perms.hasPermission(p, Permissions.SCRIPT_ERROR))
                {
                    cc.addPlayer(p);
                }
            }, 
            (cc, p) -> 
            {
                cc.removePlayer(p);
            });   
    
    public static void onPlayerLogIn(EntityPlayer p)
    {
        DEV_CHANNEL.onLogin.accept(DEV_CHANNEL, p);
    }
    
    public static void onPlayerLogOut(EntityPlayer p)
    {
        DEV_CHANNEL.onLogout.accept(DEV_CHANNEL, p);
    }
    
    // -------------------------------------------------------------------------
    // chat channels
    // -------------------------------------------------------------------------
    
    private final HashSet<EntityPlayer> receivers;
    private final String prefix;
    private final String list;
    private final BiConsumer<ChatChannel, String> onSend;
    private final BiConsumer<ChatChannel, EntityPlayer> onLogin;
    private final BiConsumer<ChatChannel, EntityPlayer> onLogout;
    
    private ChatChannel(String name, TextColor tc,
            BiConsumer<ChatChannel, String> onSend,
            BiConsumer<ChatChannel, EntityPlayer> onLogin,
            BiConsumer<ChatChannel, EntityPlayer> onLogout)
    {
        receivers = new HashSet<>();
        this.prefix = "[" + tc + name + TextColor.RESET + "] ";
        this.list = tc + " - ";
        this.onSend = onSend;
        this.onLogin = onLogin;
        this.onLogout = onLogout;
    }
    
    public void addPlayer(EntityPlayer p)
    {
        receivers.add(p);
    }
    
    public void removePlayer(EntityPlayer p)
    {
        receivers.remove(p);
    }
    
    public void send(String msg)
    {
        onSend.accept(this, prefix + " " + msg);
    }
    
    public void sendWarning(String msg)
    {
        onSend.accept(this, prefix + " " + TextColor.RED + msg);
    }
    
    public void sendList(String msg)
    {
        onSend.accept(this, list + TextColor.RESET + msg);
    }
    
    public void sendList(String msg, String msg2)
    {
        onSend.accept(this, list + msg + TextColor.RESET + " " + msg2);
    }
    
    public void forEach(Consumer<EntityPlayer> c)
    {
        receivers.forEach(c);
    }
}
